@@ -4,7 +4,7 @@ use 5.006001;
 use strict;
 use warnings;
 
-our $VERSION = '1.999807';
+our $VERSION = '1.999809';
 
 use Carp;
 
@@ -237,7 +237,7 @@ use overload
                 return $class -> _sqrt($class -> _copy($_[0]));
             },
 
-  'int'  => sub { $_[0] -> copy() -> bint(); },
+  'int'  => sub { $_[0] },
 
   # overload key: conversion
 
@@ -585,6 +585,25 @@ sub _fac {
     return $x;
 }
 
+sub _dfac {
+    # double factorial
+    my ($class, $x) = @_;
+
+    my $two = $class -> _two();
+
+    if ($class -> _acmp($x, $two) < 0) {
+        return $class -> _one();
+    }
+
+    my $i = $class -> _copy($x);
+    while ($class -> _acmp($i, $two) > 0) {
+        $i = $class -> _sub($i, $two);
+        $x = $class -> _mul($x, $i);
+    }
+
+    return $x;
+}
+
 sub _log_int {
     # calculate integer log of $x to base $base
     # calculate integer log of $x to base $base
@@ -1062,7 +1081,7 @@ sub _to_bin {
     my ($class, $x) = @_;
     my $str    = '';
     my $tmp    = $class -> _copy($x);
-    my $chunk = $class -> _new("16777216");
+    my $chunk = $class -> _new("16777216");     # 2^24 = 24 binary digits
     my $rem;
     until ($class -> _acmp($tmp, $chunk) < 0) {
         ($tmp, $rem) = $class -> _div($tmp, $chunk);
@@ -1079,7 +1098,7 @@ sub _to_oct {
     my ($class, $x) = @_;
     my $str    = '';
     my $tmp    = $class -> _copy($x);
-    my $chunk = $class -> _new("16777216");    # 2^24 / 8 octal digits
+    my $chunk = $class -> _new("16777216");     # 2^24 = 8 octal digits
     my $rem;
     until ($class -> _acmp($tmp, $chunk) < 0) {
         ($tmp, $rem) = $class -> _div($tmp, $chunk);
@@ -1096,7 +1115,7 @@ sub _to_hex {
     my ($class, $x) = @_;
     my $str    = '';
     my $tmp    = $class -> _copy($x);
-    my $chunk = $class -> _new("16777216");
+    my $chunk = $class -> _new("16777216");     # 2^24 = 6 hexadecimal digits
     my $rem;
     until ($class -> _acmp($tmp, $chunk) < 0) {
         ($tmp, $rem) = $class -> _div($tmp, $chunk);
@@ -1143,46 +1162,92 @@ sub _to_bytes {
 
 *_as_bytes = \&_to_bytes;
 
-sub _from_oct {
-    # convert a string of octal digits to a number
-    my ($class, $str) = @_;
-    $str =~ s/^0+//;
-    my $x    = $class -> _zero();
-    my $base = $class -> _new("8");
-    my $n    = length($str);
-    for (my $i = 0 ; $i < $n ; ++$i) {
-        $x = $class -> _mul($x, $base);
-        $x = $class -> _add($x, $class -> _new(substr($str, $i, 1)));
+sub _from_hex {
+    # Convert a string of hexadecimal digits to a number.
+
+    my ($class, $hex) = @_;
+    $hex =~ s/^0[xX]//;
+
+    # Find the largest number of hexadecimal digits that we can safely use with
+    # 32 bit integers. There are 4 bits pr hexadecimal digit, and we use only
+    # 31 bits to play safe. This gives us int(31 / 4) = 7.
+
+    my $len = length $hex;
+    my $rem = 1 + ($len - 1) % 7;
+
+    # Do the first chunk.
+
+    my $ret = $class -> _new(int hex substr $hex, 0, $rem);
+    return $ret if $rem == $len;
+
+    # Do the remaining chunks, if any.
+
+    my $shift = $class -> _new(1 << (4 * 7));
+    for (my $offset = $rem ; $offset < $len ; $offset += 7) {
+        my $part = int hex substr $hex, $offset, 7;
+        $ret = $class -> _mul($ret, $shift);
+        $ret = $class -> _add($ret, $class -> _new($part));
     }
-    return $x;
+
+    return $ret;
 }
 
-sub _from_hex {
-    # convert a string of hexadecimal digits to a number
-    my ($class, $str) = @_;
-    $str =~ s/^0[Xx]//;
-    my $x    = $class -> _zero();
-    my $base = $class -> _new("16");
-    my $n    = length($str);
-    for (my $i = 0 ; $i < $n ; ++$i) {
-        $x = $class -> _mul($x, $base);
-        $x = $class -> _add($x, $class -> _new(hex substr($str, $i, 1)));
+sub _from_oct {
+    # Convert a string of octal digits to a number.
+
+    my ($class, $oct) = @_;
+
+    # Find the largest number of octal digits that we can safely use with 32
+    # bit integers. There are 3 bits pr octal digit, and we use only 31 bits to
+    # play safe. This gives us int(31 / 3) = 10.
+
+    my $len = length $oct;
+    my $rem = 1 + ($len - 1) % 10;
+
+    # Do the first chunk.
+
+    my $ret = $class -> _new(int oct substr $oct, 0, $rem);
+    return $ret if $rem == $len;
+
+    # Do the remaining chunks, if any.
+
+    my $shift = $class -> _new(1 << (3 * 10));
+    for (my $offset = $rem ; $offset < $len ; $offset += 10) {
+        my $part = int oct substr $oct, $offset, 10;
+        $ret = $class -> _mul($ret, $shift);
+        $ret = $class -> _add($ret, $class -> _new($part));
     }
-    return $x;
+
+    return $ret;
 }
 
 sub _from_bin {
-    # convert a string of binary digits to a number
-    my ($class, $str) = @_;
-    $str =~ s/^0[Bb]//;
-    my $x    = $class -> _zero();
-    my $base = $class -> _new("2");
-    my $n    = length($str);
-    for (my $i = 0 ; $i < $n ; ++$i) {
-        $x = $class -> _mul($x, $base);
-        $x = $class -> _add($x, $class -> _new(substr($str, $i, 1)));
+    # Convert a string of binary digits to a number.
+
+    my ($class, $bin) = @_;
+    $bin =~ s/^0[bB]//;
+
+    # The largest number of binary digits that we can safely use with 32 bit
+    # integers is 31. We use only 31 bits to play safe.
+
+    my $len = length $bin;
+    my $rem = 1 + ($len - 1) % 31;
+
+    # Do the first chunk.
+
+    my $ret = $class -> _new(int oct '0b' . substr $bin, 0, $rem);
+    return $ret if $rem == $len;
+
+    # Do the remaining chunks, if any.
+
+    my $shift = $class -> _new(1 << 31);
+    for (my $offset = $rem ; $offset < $len ; $offset += 31) {
+        my $part = int oct '0b' . substr $bin, $offset, 31;
+        $ret = $class -> _mul($ret, $shift);
+        $ret = $class -> _add($ret, $class -> _new($part));
     }
-    return $x;
+
+    return $ret;
 }
 
 sub _from_bytes {
@@ -1340,6 +1405,100 @@ sub _lcm {
     return $x;
 }
 
+sub _lucas {
+    my ($class, $n) = @_;
+
+    $n = $class -> _num($n) if ref $n;
+
+    # In list context, use lucas(n) = lucas(n-1) + lucas(n-2)
+
+    if (wantarray) {
+        my @y;
+
+        push @y, $class -> _two();
+        return @y if $n == 0;
+
+        push @y, $class -> _one();
+        return @y if $n == 1;
+
+        for (my $i = 2 ; $i <= $n ; ++ $i) {
+            $y[$i] = $class -> _add($class -> _copy($y[$i - 1]), $y[$i - 2]);
+        }
+
+        return @y;
+    }
+
+    require Scalar::Util;
+
+    # In scalar context use that lucas(n) = fib(n-1) + fib(n+1).
+    #
+    # Remember that _fib() behaves differently in scalar context and list
+    # context, so we must add scalar() to get the desired behaviour.
+
+    return $class -> _two() if $n == 0;
+
+    return $class -> _add(scalar $class -> _fib($n - 1),
+                          scalar $class -> _fib($n + 1));
+}
+
+sub _fib {
+    my ($class, $n) = @_;
+
+    $n = $class -> _num($n) if ref $n;
+
+    # In list context, use fib(n) = fib(n-1) + fib(n-2)
+
+    if (wantarray) {
+        my @y;
+
+        push @y, $class -> _zero();
+        return @y if $n == 0;
+
+        push @y, $class -> _one();
+        return @y if $n == 1;
+
+        for (my $i = 2 ; $i <= $n ; ++ $i) {
+            $y[$i] = $class -> _add($class -> _copy($y[$i - 1]), $y[$i - 2]);
+        }
+
+        return @y;
+    }
+
+    # In scalar context use a fast algorithm that is much faster than the
+    # recursive algorith used in list context.
+
+    my $cache = {};
+    my $two = $class -> _two();
+    my $fib;
+
+    $fib = sub {
+        my $n = shift;
+        return $class -> _zero() if $n <= 0;
+        return $class -> _one()  if $n <= 2;
+        return $cache -> {$n}    if exists $cache -> {$n};
+
+        my $k = int($n / 2);
+        my $a = $fib -> ($k + 1);
+        my $b = $fib -> ($k);
+        my $y;
+
+        if ($n % 2 == 1) {
+            # a*a + b*b
+            $y = $class -> _add($class -> _mul($class -> _copy($a), $a),
+                                $class -> _mul($class -> _copy($b), $b));
+        } else {
+            # (2*a - b)*b
+            $y = $class -> _mul($class -> _sub($class -> _mul(
+                   $class -> _copy($two), $a), $b), $b);
+        }
+
+        $cache -> {$n} = $y;
+        return $y;
+    };
+
+    return $fib -> ($n);
+}
+
 ##############################################################################
 ##############################################################################
 
@@ -1527,8 +1686,15 @@ Returns the Nth root of OBJ, truncated to an integer.
 
 =item CLASS-E<gt>_fac(OBJ)
 
-Returns factorial of OBJ, i.e., the product of all positive integers up to and
-including OBJ.
+Returns the factorial of OBJ, i.e., the product of all positive integers up to
+and including OBJ.
+
+=item CLASS-E<gt>_dfac(OBJ)
+
+Returns the double factorial of OBJ. If OBJ is an even integer, returns the
+product of all positive, even integers up to and including OBJ, i.e.,
+2*4*6*...*OBJ. If OBJ is an odd integer, returns the product of all positive,
+odd integers, i.e., 1*3*5*...*OBJ.
 
 =item CLASS-E<gt>_pow(OBJ1, OBJ2)
 
@@ -1583,7 +1749,19 @@ Returns the greatest common divisor of OBJ1 and OBJ2.
 
 =item CLASS-E<gt>_lcm(OBJ1, OBJ2)
 
-Rseturn the least common multiple of OBJ1 and OBJ2.
+Return the least common multiple of OBJ1 and OBJ2.
+
+=item CLASS-E<gt>_fib(OBJ)
+
+In scalar context, returns the nth Fibonacci number: _fib(0) returns 0, _fib(1)
+returns 1, _fib(2) returns 1, _fib(3) returns 2 etc. In list context, returns
+the Fibonacci numbers from F(0) to F(n): 0, 1, 1, 2, 3, 5, 8, 13, 21, 34, ...
+
+=item CLASS-E<gt>_lucas(OBJ)
+
+In scalar context, returns the nth Lucas number: _lucas(0) returns 2, _lucas(1)
+returns 1, _lucas(2) returns 3, etc. In list context, returns the Lucas numbers
+from L(0) to L(n): 2, 1, 3, 4, 7, 11, 18, 29,47, 76, ...
 
 =back
 
@@ -1653,32 +1831,32 @@ should have no leading zeros, i.e., it should match C<^(0|[1-9]\d*)$>.
 
 Returns the binary string representation of OBJ.
 
-=item CLASS-E<gt>_as_bin(OBJ)
-
-Like C<_to_bin()> but with a '0b' prefix.
-
 =item CLASS-E<gt>_to_oct(OBJ)
 
 Returns the octal string representation of the number.
 
-=item CLASS-E<gt>_as_oct(OBJ)
-
-Like C<_to_oct()> but with a '0' prefix.
-
 =item CLASS-E<gt>_to_hex(OBJ)
 
 Returns the hexadecimal string representation of the number.
 
-=item CLASS-E<gt>_as_hex(OBJ)
-
-Like C<_to_hex()> but with a '0x' prefix.
-
 =item CLASS-E<gt>_to_bytes(OBJ)
 
 Returns a byte string representation of OBJ. The byte string is in big endian
 byte order, so if OBJ represents the number 256, the output should be the
 two-byte string "\x01\x00".
 
+=item CLASS-E<gt>_as_bin(OBJ)
+
+Like C<_to_bin()> but with a '0b' prefix.
+
+=item CLASS-E<gt>_as_oct(OBJ)
+
+Like C<_to_oct()> but with a '0' prefix.
+
+=item CLASS-E<gt>_as_hex(OBJ)
+
+Like C<_to_hex()> but with a '0x' prefix.
+
 =item CLASS-E<gt>_as_bytes(OBJ)
 
 This is an alias to C<_to_bytes()>.
